<?php

declare(strict_types=1);

namespace App\Controller\OgAgent;

use App\Dto\Input\DeployImageInput;
use App\Entity\Client;
use App\Entity\Command;
use App\Entity\Image;
use App\Entity\Trace;
use App\Model\ClientStatus;
use App\Model\TraceStatus;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class DeployImageAction extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface $entityManager,
        protected readonly HttpClientInterface    $httpClient,
        protected readonly CreateService          $createService,
    )
    {
    }


    public function __invoke(Image $image, DeployImageInput $input, string $method)
    {
        if (!$image->getClient()->getIp()) {
            throw new ValidatorException('IP is required');
        }

        $partitionInfo = json_decode($image->getPartitionInfo(), true);

        /** @var Client $client */
        $client = $input->client->getEntity();

        $data = [
            'dsk' => (string) $partitionInfo['numDisk'],
            'par' => (string) $partitionInfo['numPartition'],
            'ifs' => "1",
            'idi' => $image->getUuid(),
            'nci' => $image->getName(),
            'ipr' => $image->getRepository()->getIp(),
            'nfn' => 'RestaurarImagen',
            'ptc' => $method,
            'ids' => '0'
        ];

        try {
            $response = $this->httpClient->request('POST', 'https://'.$client->getIp().':8000/CloningEngine/RestaurarImagen', [
                'verify_peer' => false,
                'verify_host' => false,
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
                'json' => $data,
            ]);

        } catch (TransportExceptionInterface $e) {
            return new JsonResponse(
                data: ['error' => $e->getMessage()],
                status: Response::HTTP_INTERNAL_SERVER_ERROR
            );
        }

        $jobId = json_decode($response->getContent(), true)['job_id'];

        $client->setStatus(ClientStatus::BUSY);
        $this->entityManager->persist($client);
        $this->entityManager->flush();

        return $jobId;
    }
}
